@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2009 XNeur Team
+ *  Copyright (C) 2006-2010 XNeur Team
  *
  */
 
@@ -25,10 +25,12 @@
 #include <gdk/gdkx.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <gconf/gconf-client.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include <stddef.h>
+#include <unistd.h>
 #include <dirent.h>
 #include <dlfcn.h>
 
@@ -36,17 +38,16 @@
 #include <xneur/list_char.h>
 #include <xneur/xneur.h>
 
-#define GLADE_FILE_ABOUT PACKAGE_GLADE_FILE_DIR"/about.glade"
-#define GLADE_FILE_CONFIG PACKAGE_GLADE_FILE_DIR"/config.glade"
-#define GLADE_FILE_ABBREVIATION_ADD PACKAGE_GLADE_FILE_DIR"/abbr_add.glade"
-#define GLADE_FILE_CHOOSE PACKAGE_GLADE_FILE_DIR"/choose_file.glade"
-#define GLADE_FILE_ACTION_ADD PACKAGE_GLADE_FILE_DIR"/action_add.glade"
-#define GLADE_FILE_APP_ADD PACKAGE_GLADE_FILE_DIR"/app_add.glade"
-#define GLADE_FILE_LIST PACKAGE_GLADE_FILE_DIR"/list.glade"
-
 #define LANGUAGES_DIR "languages"
 #define DIR_SEPARATOR		"/"
 
+#define AUTOSTART_PATH ".config/autostart"  
+#define AUTOSTART_FILE PACKAGE ".desktop"
+
+#define GXNEUR_DESKTOP "[Desktop Entry]\nType=Application\nExec="  PACKAGE "\nHidden=false\nX-GNOME-Autostart-enabled=true\nName=GTK UI for X Neural Switcher\n"
+
+#define DEFAULT_MAX_PATH	4096
+
 #include "support.h"
 #include "callbacks.h"
 #include "trayicon.h"
@@ -54,11 +55,12 @@
 #include "misc.h"
 
 void xneur_edit_regexp(GtkWidget *treeview);
+void xneur_edit_rule(GtkWidget *treeview);
 void xneur_edit_dictionary(GtkWidget *treeview);
 
 #define MAX_LANGUAGES			4
-#define XNEUR_NEEDED_MAJOR_VERSION	9
-#define XNEUR_BUILD_MINOR_VERSION	9
+#define XNEUR_NEEDED_MAJOR_VERSION	12
+#define XNEUR_BUILD_MINOR_VERSION	0
 	
 struct _xneur_config *xconfig				= NULL;
 	
@@ -79,6 +81,8 @@ static GtkListStore *store_language			= NULL;
 
 static GtkWidget *tmp_widget	= NULL;
 
+const char *conditions_names[]				= {"contains", "begins", "ends", "coincides"};
+
 static const char *modifier_names[]			= {"Shift", "Control", "Alt", "Super"};
 static const char *all_modifiers[]			= {"Control", "Shift", "Alt", "Super", "Control_R", "Shift_R", "Alt_R", "Super_R", "Control_L", "Shift_L", "Alt_L", "Super_L"};
 
@@ -92,16 +96,18 @@ static const char *notify_names[]			=   {
 										"Correct selected text", "Transliterate selected text", "Change case of selected text", "Preview correction of selected text",
 										"Correct clipboard text", "Transliterate clipboard text", "Change case of clipboard text", "Preview correction of clipboard text",
 	                                    "Expand abbreviations",
-										"Correct aCCIDENTAL caps", "Correct TWo INitial caps", "Execute user action"
+										"Correct aCCIDENTAL caps", "Correct TWo INitial caps", "Execute user action", 
+										"Block keyboard and mouse events", "Unblock keyboard and mouse events"
 										};
 
 static const char *hotkey_names[]			=   {
-										"Correct/Undo correction","Transliterate", "Change case", "Preview correction", 
+										"Correct/Undo correction", "Transliterate", "Change case", "Preview correction", 
 										"Correct last line", "Switch between processing modes", 
 										"Correct selected text", "Transliterate selected text", "Change case of selected text", "Preview correction of selected text",
 	                                    "Correct clipboard text", "Transliterate clipboard text", "Change case of clipboard text", "Preview correction of clipboard text",
 										"Switch to layout 1", "Switch to layout 2", "Switch to layout 3", "Switch to layout 4",
-		                                "Rotate layouts", "Expand abbreviations", "Autocomplementation confirmation"
+		                                "Rotate layouts", "Rotate layouts back", "Expand abbreviations", "Autocomplementation confirmation", 
+										"Block/Unblock keyboard and mouse events", "Insert date"
                                         };
 
 static const int total_notify_names = sizeof(notify_names) / sizeof(notify_names[0]);
@@ -109,16 +115,14 @@ static const int total_notify_names = sizeof(notify_names) / sizeof(notify_names
 static const int total_modifiers			= sizeof(modifier_names) / sizeof(modifier_names[0]); 
 static const int total_all_modifiers			= sizeof(all_modifiers) / sizeof(all_modifiers[0]);
 
-static void error_msg(const char *msg, ...)
+void error_msg(const char *msg, ...)
 {
-	int len = strlen(msg) + 2;
-
 	va_list ap;
 	va_start(ap, msg);
 	
-	char *buffer = (char *) malloc(1024);
+	char *buffer = (char *) malloc(sizeof(char)*1024*10);
+	bzero(buffer, sizeof(char)*1024*10); 
 	vsprintf(buffer, msg, ap);
-	buffer[len] = 0;
 
 	GtkWidget *dialog = gtk_message_dialog_new (NULL,
 											GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -217,6 +221,35 @@ static void get_xprop_name(GladeXML *gxml)
 	gtk_entry_set_text(GTK_ENTRY(entry1), p);
 }
 
+static void get_logfile(GladeXML *gxml)
+{
+	if (gxml) {};
+	
+	char *log_home_path	= xconfig->get_home_dict_path(NULL, "xneurlog.html");
+	char *command = malloc ((strlen("xdg-open ") + strlen(log_home_path) + strlen(" 2> /dev/stdout") + 1) * sizeof(char));
+	command[0] = '\0';
+	strcat(command, "xdg-open ");
+	strcat(command, log_home_path);
+	strcat(command, " 2> /dev/stdout");
+
+	FILE *fp = popen(command, "r");
+	free(log_home_path);
+	free(command);
+	if (fp == NULL)
+		return;
+
+	char buffer[NAME_MAX];
+	if (fgets(buffer, NAME_MAX, fp) == NULL)
+	{
+		pclose(fp);
+		return;
+	}
+
+	pclose(fp);
+	
+	error_msg(buffer);
+}
+
 static void xneur_insert_application(GladeXML *gxml)
 {
 	GtkWidget *entry1 = glade_xml_get_widget (gxml, "entry1");
@@ -305,8 +338,8 @@ static void init_libxnconfig(void)
 
 	if (!xconfig->load(xconfig))
 	{
-		error_msg(_("XNeur config broken!\nPlease, remove ~/.xneur/xneurrc and reinstall XNeur package!\n"));
-		printf(_("XNeur config broken!\nPlease, remove ~/.xneur/xneurrc and reinstall XNeur package!\n"));
+		error_msg(_("XNeur's config broken or was created with old version!\nPlease, remove ~/.xneur/. It should solve the problem!\nIf you don't want to loose your configuration, back it up\nand manually patch new configuration which will be created after first run.\n"));
+		printf(_("XNeur's config broken or was created with old version!\nPlease, remove ~/.xneur/. It should solve the problem!\nIf you don't want to loose your configuration, back it up\nand manually patch new configuration which will be created after first run.\n"));
 		xconfig->uninit(xconfig);
 		exit(EXIT_FAILURE);
 	}
@@ -318,38 +351,11 @@ void xneur_start(void)
 
 	if (!g_spawn_command_line_async("xneur", NULL))
 	{
-		error_msg(_("Couldn't start xneur\nVerify that it installed\n"));
-		fprintf(stderr, _("Couldn't start xneur\nVerify that it installed\n"));
+		error_msg(_("Couldn't start %s\nVerify that it installed\n"), "xneur");
+		fprintf(stderr, _("Couldn't start %s\nVerify that it installed\n"), "xneur");
 	}
 }
 
-void xneur_exit(void)
-{
-	xconfig->kill(xconfig);
-	gtk_main_quit();
-}
-
-void xneur_start_stop(GtkWidget *widget, struct _tray_icon *tray)
-{
-	if (widget){};
-
-	if (xconfig->kill(xconfig) == TRUE)
-	{
-		create_tray_icon(tray, FALSE);
-		return;
-	}
-
-	xneur_start();
-	create_tray_icon(tray, TRUE);
-}
-
-void xneur_auto_manual(GtkWidget *widget, struct _tray_icon *tray)
-{
-	if (widget || tray){};
-
-	xconfig->set_manual_mode(xconfig, !xconfig->is_manual_mode(xconfig));
-}
-
 void xneur_about(void)
 { 
 	GladeXML *gxml = glade_xml_new (GLADE_FILE_ABOUT, NULL, NULL);
@@ -397,6 +403,16 @@ static void column_1_edited (GtkCellRendererText *renderer, gchar *path, gchar *
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 1, new_text, -1);
 }
 
+static void column_2_edited (GtkCellRendererText *renderer, gchar *path, gchar *new_text, GtkTreeView *treeview)
+{
+	if (renderer) {};
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	model = gtk_tree_view_get_model (treeview);
+	if (gtk_tree_model_get_iter_from_string (model, &iter, path))
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, new_text, -1);
+}
 static void plug_enable (GtkCellRendererToggle *renderer, gchar *path, GtkTreeView *treeview)
 {
 	GtkTreeIter iter;
@@ -421,6 +437,109 @@ static void notify_enable (GtkCellRendererToggle *renderer, gchar *path, GtkTree
 		gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, value, -1);
 }
 
+static void xneur_replace_pixmap_dir(GladeXML *gxml)
+{
+	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
+
+	gtk_entry_set_text(GTK_ENTRY(tmp_widget), gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (window)));
+	
+	gtk_widget_destroy(window);
+}
+
+static void xneur_restore_pixmap_dir(GladeXML *gxml)
+{
+	GtkWidget *widget = glade_xml_get_widget (gxml, "entry1");
+
+	gtk_entry_set_text(GTK_ENTRY(widget), PACKAGE_PIXMAPS_DIR);
+}
+
+static void xneur_edit_pixmap_dir(GladeXML *parent_gxml)
+{
+	GladeXML *gxml = glade_xml_new (GLADE_FILE_CHOOSE, NULL, NULL);
+	
+	glade_xml_signal_autoconnect (gxml);
+	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
+
+	tmp_widget = glade_xml_get_widget (parent_gxml, "entry1");
+	
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(window), gtk_entry_get_text(GTK_ENTRY(tmp_widget)));
+	gtk_file_chooser_set_action(GTK_FILE_CHOOSER(window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);	
+	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
+	if (window_icon_pixbuf)
+	{
+		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
+		gdk_pixbuf_unref (window_icon_pixbuf);
+	}
+	
+	gtk_widget_show(window);
+		
+	// Button OK
+	GtkWidget *widget = glade_xml_get_widget (gxml, "button5");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_replace_pixmap_dir), gxml);
+}
+
+static void xneur_restore_keyboard_properties(GladeXML *gxml)
+{
+	GtkWidget *widget = glade_xml_get_widget (gxml, "entry5");
+
+	gtk_entry_set_text(GTK_ENTRY(widget), KB_PROP_COMMAND);
+}
+
+static void xneur_replace_keyboard_properties(GladeXML *gxml)
+{
+	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
+
+	gtk_entry_set_text(GTK_ENTRY(tmp_widget), gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (window)));
+	
+	gtk_widget_destroy(window);
+}	
+
+static void xneur_edit_keyboard_properties(GladeXML *parent_gxml)
+{
+	GladeXML *gxml = glade_xml_new (GLADE_FILE_CHOOSE, NULL, NULL);
+	
+	glade_xml_signal_autoconnect (gxml);
+	GtkWidget *window = glade_xml_get_widget (gxml, "filechooserdialog1");
+
+	tmp_widget = glade_xml_get_widget (parent_gxml, "entry5");
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(window), gtk_entry_get_text(GTK_ENTRY(tmp_widget)));
+	GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
+	if (window_icon_pixbuf)
+	{
+		gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
+		gdk_pixbuf_unref (window_icon_pixbuf);
+	}
+	
+	gtk_widget_show(window);
+		
+	// Button OK
+	GtkWidget *widget = glade_xml_get_widget (gxml, "button5");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_replace_keyboard_properties), gxml);
+}
+
+void xneur_kb_preference(void)
+{
+	GConfClient* gconfClient = gconf_client_get_default();
+	g_assert(GCONF_IS_CLIENT(gconfClient));
+
+	GConfValue* gcValue = NULL;
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", NULL);
+	if(gcValue != NULL) 
+	{
+		const char *string_value = NULL;
+		if(gcValue->type == GCONF_VALUE_STRING) 
+		{
+			string_value = gconf_value_get_string(gcValue);
+			if (!g_spawn_command_line_async(string_value, NULL))
+			{
+				error_msg(_("Couldn't start %s\nVerify that it installed\n"), string_value);
+				fprintf(stderr, _("Couldn't start %s\nVerify that it installed\n"), string_value);
+			}
+		}
+		gconf_value_free(gcValue);
+	}
+}
+
 void xneur_preference(void)
 {
 	GladeXML *gxml = glade_xml_new (GLADE_FILE_CONFIG, NULL, NULL);
@@ -583,9 +702,6 @@ void xneur_preference(void)
 												-1);
 	}
 
-	widget = glade_xml_get_widget (gxml, "button7");
-	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_regexp), G_OBJECT(treeview));
-
 	widget = glade_xml_get_widget (gxml, "button6");
 	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_dictionary), G_OBJECT(treeview));
 
@@ -603,7 +719,11 @@ void xneur_preference(void)
 	
 	// Saving Selection Mode
 	widget = glade_xml_get_widget (gxml, "checkbutton4");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->save_selection);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->save_selection_after_convert);
+
+	// Rotate Layout after correction selected text Mode
+	widget = glade_xml_get_widget (gxml, "checkbutton30");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->rotate_layout_after_convert);
 	
 	// Sound Playing Mode
 	widget = glade_xml_get_widget (gxml, "checkbutton5");
@@ -622,11 +742,21 @@ void xneur_preference(void)
 
 	// Log send to e-mail
 	widget = glade_xml_get_widget (gxml, "entry3");
-	gtk_entry_set_text(GTK_ENTRY(widget), xconfig->mail_keyboard_log);
+	if (xconfig->mail_keyboard_log != NULL)
+		gtk_entry_set_text(GTK_ENTRY(widget), xconfig->mail_keyboard_log);
 	
 	// Log send via host
 	widget = glade_xml_get_widget (gxml, "entry4");
-	gtk_entry_set_text(GTK_ENTRY(widget), xconfig->host_keyboard_log);
+	if (xconfig->host_keyboard_log != NULL)
+		gtk_entry_set_text(GTK_ENTRY(widget), xconfig->host_keyboard_log);
+
+	// Log port
+	widget = glade_xml_get_widget (gxml, "spinbutton4");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), xconfig->port_keyboard_log);
+
+	// View log
+	widget = glade_xml_get_widget (gxml, "button8");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(get_logfile), gxml);
 	
 	// Ignore Keyboard Layout Mode
 	widget = glade_xml_get_widget (gxml, "checkbutton8");
@@ -644,6 +774,10 @@ void xneur_preference(void)
 	widget = glade_xml_get_widget (gxml, "checkbutton20");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_space_with_punctuation);
 
+	// 
+	widget = glade_xml_get_widget (gxml, "checkbutton31");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_capital_letter_after_dot);
+
 	// Autocomplementation
 	widget = glade_xml_get_widget (gxml, "checkbutton21");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->autocomplementation);
@@ -707,11 +841,15 @@ void xneur_preference(void)
 	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 400);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), GTK_TREE_VIEW_COLUMN(column));
-
+	
 	cell = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Key bind"), cell, "text", 1, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), GTK_TREE_VIEW_COLUMN(column));
 
+	g_signal_connect (G_OBJECT (treeview), "row-activated",
+						G_CALLBACK (xneur_edit_action),
+						(gpointer) treeview);
+	
 	// Button Edit Action
 	widget = glade_xml_get_widget (gxml, "button10");
 	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_action), G_OBJECT(treeview));
@@ -807,6 +945,10 @@ void xneur_preference(void)
 												-1);
 	}
 
+	g_signal_connect (G_OBJECT (treeview), "row-activated",
+						G_CALLBACK (xneur_edit_sound),
+						(gpointer) treeview);
+	                  
 	// Button Edit Sound
 	widget = glade_xml_get_widget (gxml, "button12");
 	tmp_widget = GTK_WIDGET(treeview);
@@ -815,7 +957,7 @@ void xneur_preference(void)
 	// Delay Before Send
 	widget = glade_xml_get_widget (gxml, "spinbutton1");
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), xconfig->send_delay);
-		
+	
 	// Log Level
 	widget = glade_xml_get_widget (gxml, "combobox1");
 	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), xconfig->log_level);
@@ -828,10 +970,18 @@ void xneur_preference(void)
 	widget = glade_xml_get_widget (gxml, "checkbutton9");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->correct_incidental_caps);
 
+	// Flush internal buffer when pressed Escape mode
+	widget = glade_xml_get_widget (gxml, "checkbutton32");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->flush_buffer_when_press_escape);
+
 	// Flush internal buffer when pressed Enter mode
 	widget = glade_xml_get_widget (gxml, "checkbutton11");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->flush_buffer_when_press_enter);
 
+	// Compatibility with completion
+	widget = glade_xml_get_widget (gxml, "checkbutton33");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->compatibility_with_completion);
+	
 	// Don't process word when pressed Enter mode
 	widget = glade_xml_get_widget (gxml, "checkbutton12");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->dont_process_when_press_enter);
@@ -875,7 +1025,7 @@ void xneur_preference(void)
 	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), True);
 	g_object_set (cell, "editable", TRUE, "editable-set", TRUE, NULL);
 	g_signal_connect (G_OBJECT (cell), "edited",
-						G_CALLBACK (column_1_edited),
+						G_CALLBACK (column_0_edited),
 						(gpointer) treeview);
 	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 200);
@@ -889,12 +1039,16 @@ void xneur_preference(void)
 	column = gtk_tree_view_column_new_with_attributes(_("User action"), cell, "text", 2, NULL);
 	g_object_set (cell, "editable", TRUE, "editable-set", TRUE, NULL);
 	g_signal_connect (G_OBJECT (cell), "edited",
-						G_CALLBACK (column_1_edited),
+						G_CALLBACK (column_2_edited),
 						(gpointer) treeview);
 	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), True);
 	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column), GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 200);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), GTK_TREE_VIEW_COLUMN(column));
+
+	g_signal_connect (G_OBJECT (treeview), "row-activated",
+						G_CALLBACK (xneur_edit_user_action),
+						(gpointer) treeview);
 	
 	// Button Add User Action
 	widget = glade_xml_get_widget (gxml, "button36");
@@ -1019,6 +1173,10 @@ void xneur_preference(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->troubleshoot_delete);
 	widget = glade_xml_get_widget (gxml, "checkbutton25");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->troubleshoot_switch);
+
+	// Dont Send KeyRelease Mode
+	widget = glade_xml_get_widget (gxml, "checkbutton26");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), xconfig->dont_send_key_release);
 	
 	// Plugins
 	treeview = glade_xml_get_widget (gxml, "treeview11");
@@ -1071,9 +1229,6 @@ void xneur_preference(void)
 				continue;
 			}
 
-			char* (*module_info)(void);
-			module_info = dlsym(module, "on_plugin_info");
-
 			gboolean state = FALSE;
 			for (int i = 0; i < xconfig->plugins->data_count; i++)
 			{
@@ -1083,20 +1238,139 @@ void xneur_preference(void)
 					break;
 				}
 			}
+
+			char* mod_info = "";
+			char* (*module_info)(void);
+			module_info = dlsym(module, "on_plugin_info");
+			if (module_info != NULL) 
+			{
+				mod_info = (gchar *)module_info();
+			}
 			
 			GtkTreeIter iter;
 			gtk_list_store_append(GTK_LIST_STORE(store_plugin), &iter);
 			gtk_list_store_set(GTK_LIST_STORE(store_plugin), &iter, 
 												0, state,
-												1, (gchar *)module_info(),
+												1, mod_info,
 												2, ep->d_name,
 												-1);
-	
+
 			ep = readdir (dp);
 			dlclose(module);
 		}
  		(void) closedir (dp);
 	}
+
+	// Gxneur Properties
+
+	// Autostart
+	gchar *path_file = g_build_filename(getenv("HOME"), AUTOSTART_PATH, AUTOSTART_FILE, NULL);
+	FILE *stream = fopen(path_file, "r");
+	g_free(path_file);
+	if (stream != NULL)
+	{
+		fclose(stream);
+		widget = glade_xml_get_widget (gxml, "checkbutton27");
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+	}
+	
+	// Delay before start
+	GConfClient* gconfClient = gconf_client_get_default();
+	g_assert(GCONF_IS_CLIENT(gconfClient));
+
+	// Delay before start
+	GConfValue* gcValue = NULL;
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "delay", NULL);
+
+	int value = 0;
+	if(gcValue != NULL) 
+	{
+		/* Check if value type is integer */
+		if(gcValue->type == GCONF_VALUE_INT) 
+			value = gconf_value_get_int(gcValue);
+
+		/* Release resources */
+		gconf_value_free(gcValue);
+	}
+
+	widget = glade_xml_get_widget (gxml, "spinbutton5");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), value);
+
+	// Text on tray
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "text_on_tray", NULL);
+
+	value = FALSE;
+	if(gcValue != NULL) 
+	{
+		if(gcValue->type == GCONF_VALUE_BOOL) 
+			value = gconf_value_get_bool(gcValue);
+
+		gconf_value_free(gcValue);
+	}		
+
+	widget = glade_xml_get_widget (gxml, "checkbutton28");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), value);
+
+	// Resize icon on tray
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon", NULL);
+
+	value = FALSE;
+	if(gcValue != NULL) 
+	{
+		if(gcValue->type == GCONF_VALUE_BOOL) 
+			value = gconf_value_get_bool(gcValue);
+
+		gconf_value_free(gcValue);
+	}		
+
+	widget = glade_xml_get_widget (gxml, "checkbutton29");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), value);
+
+	//
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir", NULL);
+
+	/* if value pointer remains NULL, the key was not found */
+	const char *string_value = NULL;
+	if(gcValue != NULL) 
+	{
+		if(gcValue->type == GCONF_VALUE_STRING) 
+		{
+			string_value = gconf_value_get_string(gcValue);
+			widget = glade_xml_get_widget (gxml, "entry1");
+			gtk_entry_set_text(GTK_ENTRY(widget), string_value);
+		}
+	}
+	
+	// Button Pixmap Directory Edit
+	widget = glade_xml_get_widget (gxml, "button9");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_pixmap_dir), gxml);
+	// Button Pixmap Directory Edit
+	widget = glade_xml_get_widget (gxml, "button14");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_restore_pixmap_dir), gxml);
+
+	
+	// Keyboard properties
+	gcValue = gconf_client_get_without_default(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", NULL);
+
+	if(gcValue != NULL) 
+	{
+		if(gcValue->type == GCONF_VALUE_STRING) 
+		{
+			string_value = gconf_value_get_string(gcValue);
+			widget = glade_xml_get_widget (gxml, "entry5");
+			gtk_entry_set_text(GTK_ENTRY(widget), string_value);
+		}
+	}
+	
+	// Button Pixmap Directory Edit
+	widget = glade_xml_get_widget (gxml, "button15");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_edit_keyboard_properties), gxml);
+	// Button Pixmap Directory Edit
+	widget = glade_xml_get_widget (gxml, "button16");
+	g_signal_connect_swapped(G_OBJECT(widget), "clicked", G_CALLBACK(xneur_restore_keyboard_properties), gxml);
+
+	/* release GConf client */
+	g_object_unref(gconfClient);
 	
 	// Button OK
 	widget = glade_xml_get_widget (gxml, "button5");
@@ -1378,6 +1652,7 @@ void xneur_edit_action(GtkWidget *treeview)
 		gtk_entry_set_text(GTK_ENTRY(widget), action);
 		
 		widget= glade_xml_get_widget (gxml, "entry2");
+		gtk_widget_grab_focus(widget);
 		g_signal_connect ((gpointer) widget, "key-press-event", G_CALLBACK (on_key_press_event), gxml);
 		g_signal_connect ((gpointer) widget, "key-release-event", G_CALLBACK (on_key_release_event), gxml);
 		gtk_entry_set_text(GTK_ENTRY(widget), key_bind);
@@ -1451,64 +1726,6 @@ void xneur_edit_sound(GtkWidget *treeview)
 	}
 }
 
-void xneur_edit_regexp(GtkWidget *treeview)
-{
-	tmp_widget = GTK_WIDGET(treeview);
-	GtkTreeModel *model = GTK_TREE_MODEL(store_language);
-	GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-
-	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
-
-	GtkTreeIter iter;
-	if (gtk_tree_selection_get_selected(select, &model, &iter))
-	{
-		char *dir;
-		gtk_tree_model_get(GTK_TREE_MODEL(store_language), &iter, 1, &dir, -1);
-		
-		GladeXML *gxml = glade_xml_new (GLADE_FILE_LIST, NULL, NULL);
-
-		int dir_len = strlen(LANGUAGES_DIR) + strlen(DIR_SEPARATOR) + strlen(dir) + 1;
-		char *dir_name = (char *) malloc(dir_len * sizeof(char));
-		snprintf(dir_name, dir_len, "%s%s%s", LANGUAGES_DIR, DIR_SEPARATOR, dir);
-		
-		char *text_path		= xconfig->get_global_dict_path(dir_name, "regexp");
-		char *text_home_path	= xconfig->get_home_dict_path(dir_name, "regexp");
-		char *text		= xneur_get_file_content(text_path);
-
-		if (text == NULL)
-		{
-			free(text_home_path);
-			free(text_path);
-			return;
-		}
-	
-		GtkWidget *window = glade_xml_get_widget (gxml, "dialog1");
-		GdkPixbuf *window_icon_pixbuf = create_pixbuf ("gxneur.png");
-		if (window_icon_pixbuf)
-		{
-			gtk_window_set_icon (GTK_WINDOW (window), window_icon_pixbuf);
-			gdk_pixbuf_unref (window_icon_pixbuf);
-		}
-		gtk_widget_show(window);
-
-		GtkWidget *widget = glade_xml_get_widget (gxml, "textview1");
-		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
-		gtk_text_buffer_set_text(buffer, text, strlen(text));
-
-		widget = glade_xml_get_widget (gxml, "entry10");
-		gtk_entry_set_text(GTK_ENTRY(widget), text_home_path);
-
-		widget= glade_xml_get_widget (gxml, "okbutton1");
-		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_okbutton1_clicked), gxml);
-		widget = glade_xml_get_widget (gxml, "cancelbutton1");
-		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_cancelbutton1_clicked), gxml);
-	
-		free(text);
-		free(text_home_path);
-		free(text_path);
-	}
-}
-
 void xneur_edit_dictionary(GtkWidget *treeview)
 {
 	tmp_widget = GTK_WIDGET(treeview);
@@ -1528,12 +1745,11 @@ void xneur_edit_dictionary(GtkWidget *treeview)
 		int dir_len = strlen(LANGUAGES_DIR) + strlen(DIR_SEPARATOR) + strlen(dir) + 1;
 		char *dir_name = (char *) malloc(dir_len * sizeof(char));
 		snprintf(dir_name, dir_len, "%s%s%s", LANGUAGES_DIR, DIR_SEPARATOR, dir);
-
-		char *text_path		= xconfig->get_global_dict_path(dir_name, "dict");
-		char *text_home_path	= xconfig->get_home_dict_path(dir_name, "dict");
+		
+		char *text_path		= xconfig->get_global_dict_path(dir_name, "dictionary");
+		char *text_home_path	= xconfig->get_home_dict_path(dir_name, "dictionary");
 		char *text		= xneur_get_file_content(text_path);
 
-		printf("%s %s\n", text_path, text_home_path);
 		if (text == NULL)
 		{
 			free(text_home_path);
@@ -1550,23 +1766,95 @@ void xneur_edit_dictionary(GtkWidget *treeview)
 		}
 		gtk_widget_show(window);
 
-		GtkWidget *widget = glade_xml_get_widget (gxml, "textview1");
-		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
-		gtk_text_buffer_set_text(buffer, text, strlen(text));
+		GtkWidget *widget = glade_xml_get_widget (gxml, "treeview1");
+		GtkListStore *store_dict = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+		gtk_tree_view_set_model(GTK_TREE_VIEW(widget), GTK_TREE_MODEL(store_dict));
+
+		GtkCellRenderer *cell = gtk_cell_renderer_text_new();
+		GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(_("Combination of letters"), cell, "text", 0, NULL);
+		//gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), True);
+		gtk_tree_view_append_column(GTK_TREE_VIEW(widget), GTK_TREE_VIEW_COLUMN(column));
+
+		cell = gtk_cell_renderer_text_new();
+		column = gtk_tree_view_column_new_with_attributes(_("Condition"), cell, "text", 1, NULL);
+		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), True);
+		gtk_tree_view_append_column(GTK_TREE_VIEW(widget), GTK_TREE_VIEW_COLUMN(column));
+
+		cell = gtk_cell_renderer_toggle_new();
+		column = gtk_tree_view_column_new_with_attributes(_("Insensitive"), cell, "active", 2, NULL);
+		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), True);
+		gtk_tree_view_append_column(GTK_TREE_VIEW(widget), GTK_TREE_VIEW_COLUMN(column));
+
+		while (1)
+		{
+			char *line = strsep(&text, "\n");
+			if (line == NULL)
+				break;
+
+			if (line[0] == '\0')
+				continue;
+
+			const char *condition = conditions_names[0];
+			gboolean insensitive = FALSE;
+			if (strncmp(line, "(?i)", 4 * sizeof(char)) == 0)
+			{
+				insensitive = TRUE;
+				memmove(line, line + 4 * sizeof(char), (strlen(line)-3) * sizeof(char)); 
+			}
+
+			if ((line[0] == '^') && (line[strlen(line)-1] == '$'))
+			{
+				condition = conditions_names[3];
+				memmove(line, line + 1 * sizeof(char), strlen(line));
+				line[strlen(line) - 1] = '\0';
+			}
+			else if (line[0] == '^')
+			{
+				condition = conditions_names[1];
+				memmove(line, line + 1 * sizeof(char), strlen(line));
+			}
+			else if (line[strlen(line)] == '$')
+			{
+				condition = conditions_names[2];
+				line[strlen(line) - 1] = '\0';
+			}
+			
+			GtkTreeIter iter;
+			gtk_list_store_append(GTK_LIST_STORE(store_dict), &iter);
+			gtk_list_store_set(GTK_LIST_STORE(store_dict), &iter, 
+												0, line,
+			                   					1, _(condition),
+			                   					2, insensitive,
+												-1);
+		}
 
 		widget = glade_xml_get_widget (gxml, "entry10");
 		gtk_entry_set_text(GTK_ENTRY(widget), text_home_path);
 
-		widget= glade_xml_get_widget (gxml, "okbutton1");
-		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_okbutton1_clicked), gxml);
-		widget = glade_xml_get_widget (gxml, "cancelbutton1");
-		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_cancelbutton1_clicked), gxml);
+		xyz_t *ud = malloc(sizeof(xyz_t));
+		ud->x = gxml;
+		ud->y = store_dict;
+
+		widget= glade_xml_get_widget (gxml, "addbutton");
+		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK(on_addbutton_clicked), ud);
+
+		widget= glade_xml_get_widget (gxml, "editbutton");
+		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK(on_editbutton_clicked), ud);
+
+		widget= glade_xml_get_widget (gxml, "deletebutton");
+		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK(on_deletebutton_clicked), ud);
+	
+		widget= glade_xml_get_widget (gxml, "okbutton");
+		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_okbutton_clicked), ud);
+
+		widget = glade_xml_get_widget (gxml, "cancelbutton");
+		g_signal_connect ((gpointer) widget, "clicked", G_CALLBACK (on_cancelbutton_clicked), ud);
 	
 		free(text);
 		free(text_home_path);
 		free(text_path);
 	}
-}
+}	
 
 void xneur_rem_exclude_app(GtkWidget *widget)
 {
@@ -1842,14 +2130,7 @@ gboolean save_language(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter
 
 	int i = atoi(gtk_tree_path_to_string(path));
 
-	xconfig->handle->languages = (struct _xneur_language *) realloc(xconfig->handle->languages, (xconfig->handle->total_languages + 1) * sizeof(struct _xneur_language));
-	bzero(&(xconfig->handle->languages[xconfig->handle->total_languages]), sizeof(struct _xneur_language));
-
-	xconfig->handle->languages[xconfig->handle->total_languages].name	= strdup(name);
-	xconfig->handle->languages[xconfig->handle->total_languages].dir	= strdup(dir);
-	xconfig->handle->languages[xconfig->handle->total_languages].group	= i;
-	xconfig->handle->languages[xconfig->handle->total_languages].excluded	= state;
-	xconfig->handle->total_languages++;
+	xconfig->handle->languages[i].excluded	= state;
 	
 	return FALSE;
 }
@@ -1858,7 +2139,6 @@ void xneur_save_preference(GladeXML *gxml)
 {
 	xconfig->clear(xconfig);
 
-	xconfig->handle->total_languages = 0;
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store_language), save_language, NULL);
 	
 	GtkWidget *widgetPtrToBefound = glade_xml_get_widget (gxml, "combobox25");
@@ -1887,7 +2167,10 @@ void xneur_save_preference(GladeXML *gxml)
 	xconfig->remember_layout = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton4");
-	xconfig->save_selection = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+	xconfig->save_selection_after_convert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+	
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton30");
+	xconfig->rotate_layout_after_convert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton5");
 	xconfig->play_sounds = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
@@ -1909,7 +2192,11 @@ void xneur_save_preference(GladeXML *gxml)
 	// Log send via host
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry4");
 	xconfig->host_keyboard_log = strdup((char *) gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
-	
+
+	// Log port
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "spinbutton4");
+	xconfig->port_keyboard_log = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound));
+
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton8");
 	xconfig->abbr_ignore_layout = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	
@@ -1927,12 +2214,18 @@ void xneur_save_preference(GladeXML *gxml)
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton9");
 	xconfig->correct_incidental_caps = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton32");
+	xconfig->flush_buffer_when_press_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton11");
 	xconfig->flush_buffer_when_press_enter = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton12");
 	xconfig->dont_process_when_press_enter = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
-	
+
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton33");
+	xconfig->compatibility_with_completion = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton13");
 	xconfig->show_osd = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
@@ -1945,6 +2238,9 @@ void xneur_save_preference(GladeXML *gxml)
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton20");
 	xconfig->correct_space_with_punctuation = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton31");
+	xconfig->correct_capital_letter_after_dot = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton21");
 	xconfig->autocomplementation = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 
@@ -1975,6 +2271,96 @@ void xneur_save_preference(GladeXML *gxml)
 	xconfig->troubleshoot_delete = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
 	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton25");
 	xconfig->troubleshoot_switch = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));;
+
+	// Dont send KeyRelease mode
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton26");
+	xconfig->dont_send_key_release = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound));
+
+	// Autostart
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton27");
+
+	gchar *path_file = g_build_filename(getenv("HOME"), AUTOSTART_PATH, NULL);
+	if (!g_file_test(path_file, G_FILE_TEST_IS_DIR))
+		g_mkdir_with_parents(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+	g_free(path_file);
+	
+	path_file = g_build_filename(getenv("HOME"), AUTOSTART_PATH, AUTOSTART_FILE, NULL);
+	FILE *stream = fopen(path_file, "r");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)))
+	{
+		if (stream != NULL)
+		{
+			fclose(stream);
+			remove(path_file);
+		}
+		// Create .desktop file
+		stream = fopen(path_file, "w");
+		if (stream != NULL)
+		{
+			fprintf(stream, GXNEUR_DESKTOP);
+			fclose(stream);
+		}
+	}
+	else
+	{
+		if (stream != NULL)
+		{
+			// delete .desktop file
+			fclose(stream);
+			remove(path_file);
+		}
+	}
+	g_free(path_file);
+
+	
+	GConfClient* gconfClient = gconf_client_get_default();
+	g_assert(GCONF_IS_CLIENT(gconfClient));
+	// Delay before start
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "spinbutton5");
+	 
+	if(!gconf_client_set_int(gconfClient, PACKAGE_GCONF_DIR "delay", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound)), NULL)) 
+	{
+	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "delay", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgetPtrToBefound)));
+	}
+
+	// Text on tray
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton28");
+	 
+	if(!gconf_client_set_bool(gconfClient, PACKAGE_GCONF_DIR "text_on_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)), NULL)) 
+	{
+	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "text_on_tray", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)));
+	}
+	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "text_on_tray");
+
+	// Resize icon on tray
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "checkbutton29");
+	 
+	if(!gconf_client_set_bool(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)), NULL)) 
+	{
+	    g_warning("Failed to set %s (%d)\n", PACKAGE_GCONF_DIR "resize_tray_icon", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (widgetPtrToBefound)));
+	}
+	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "resize_tray_icon");
+
+	// Path to pixmap dir
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry1");
+	
+	if(!gconf_client_set_string(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)), NULL)) 
+	{
+	    g_warning("Failed to set %s (%s)\n", PACKAGE_GCONF_DIR "pixmap_dir", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
+	}
+	gconf_client_notify(gconfClient, PACKAGE_GCONF_DIR "pixmap_dir");
+
+	// Keyboard properties
+	widgetPtrToBefound = glade_xml_get_widget (gxml, "entry5");
+	
+	if(!gconf_client_set_string(gconfClient, PACKAGE_GCONF_DIR "keyboard_properties", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)), NULL)) 
+	{
+	    g_warning("Failed to set %s (%s)\n", PACKAGE_GCONF_DIR "keyboard_properties", gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
+	}
+	add_pixmap_directory(gtk_entry_get_text(GTK_ENTRY(widgetPtrToBefound)));
+	
+    /* release GConf client */
+    g_object_unref(gconfClient);
 	
 	GtkWidget *window = glade_xml_get_widget (gxml, "window2");
 	gtk_widget_destroy(window);
