@@ -13,26 +13,34 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2009 XNeur Team
+ *  Copyright (C) 2006-2010 XNeur Team
  *
  */
 
 #ifndef _TRAYICON_H_
 #define _TRAYICON_H_
 
+#define MAX_LAYOUTS 4
+
 #include "tray_widget.h"
 
 struct _tray_icon
 {
-	GtkTrayIcon   *widget;
-
+	GtkTrayIcon *tray_icon;
+	
+	GdkPixbuf  *images[MAX_LAYOUTS];
+		
 	GtkTooltips *tooltip;
 	GtkWidget *image;
-	GtkWidget *popup_menu;
+	GtkWidget *tray_menu;
 	GtkWidget *evbox;
-	GtkWidget *clock;
+
+	gint height;
 };
 
-void create_tray_icon(struct _tray_icon *tray, gboolean runned);
+void create_tray_icon(void);
+void xneur_start_stop(void);
+void xneur_auto_manual(void);
+void xneur_exit(void);
 
 #endif /* _TRAYICON_H_ */
